@@ -6,6 +6,7 @@ use reqwest::header::HeaderValue;
 use serde::{Deserialize, Serialize};
 use thiserror::Error;
 
+use crate::crates::is_new_crates_format;
 use crate::crates_index::rewrite_config_json;
 
 #[derive(Error, Debug)]
@@ -29,11 +30,13 @@ pub struct ConfigRustup {
     pub sync: bool,
     pub download_threads: usize,
     pub source: String,
+    pub download_dev: Option<bool>,
     pub platforms_unix: Option<Vec<String>>,
     pub platforms_windows: Option<Vec<String>>,
     pub keep_latest_stables: Option<usize>,
     pub keep_latest_betas: Option<usize>,
     pub keep_latest_nightlies: Option<usize>,
+    pub pinned_rust_versions: Option<Vec<String>>,
 }
 
 #[derive(Serialize, Deserialize, Debug)]
@@ -42,6 +45,7 @@ pub struct ConfigCrates {
     pub download_threads: usize,
     pub source: String,
     pub source_index: String,
+    pub use_new_crates_format: Option<bool>,
     pub base_url: Option<String>,
 }
 
@@ -120,6 +124,18 @@ pub fn sync(path: &Path) -> Result<(), MirrorError> {
     }
     let mirror = load_mirror_toml(path)?;
 
+    // Fail if use_new_crates_format is not true, and old format is detected.
+    // If use_new_crates_format is true and new format is detected, warn the user.
+    // If use_new_crates_format is true, ignore the format and assume it's new.
+    if let Some(crates) = &mirror.crates {
+        if crates.sync && !is_new_crates_format(&path.join("crates"))? {
+            eprintln!("Your crates directory is using the old 0.2 format, however");
+            eprintln!("Panamax 0.3 has deprecated this format for a new one.");
+            eprintln!("Please delete crates/ from your mirror directory to continue.");
+            return Ok(());
+        }
+    }
+
     // Handle the contact information
 
     let user_agent_str = if let Some(ref contact) = mirror.mirror.contact {
