@@ -1,3 +1,4 @@
+use console::{pad_str, style};
 use indicatif::{ProgressBar, ProgressStyle};
 use std::sync::mpsc;
 use std::sync::mpsc::Sender;
@@ -56,3 +57,17 @@ pub fn progress_bar(
 
     (pb_thread, sender)
 }
+
+pub fn current_step_prefix(step: usize, steps: usize) -> String {
+    style(format!("[{}/{}]", step, steps)).bold().to_string()
+}
+
+pub fn padded_prefix_message(step: usize, steps: usize, msg: &str) -> String {
+    pad_str(
+        &format!("{} {}...", current_step_prefix(step, steps), msg),
+        34,
+        console::Alignment::Left,
+        None,
+    )
+    .to_string()
+}
